/*
PFEdemo1 - Probas

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import simu.*;

//utilise Proba et MoreProba

/**
 * Algorithme de simulation pour SimuLab.
 * TCL2
 */

//	compilation: javac -target 1.1 -bootclasspath ~/Java/jdk1.1/JDKClasses.zip -extdirs "" -encoding ISO-8859-1 -classpath .:SimuLab.jar TCL2.java

public class TCL2 extends SimuApplet {
	final static int numax = 35;
	final static int tempo = 1000; // un affichage / 1s
	final static int nGauss = 128;
	final static double fact = 3.5;
	Affichage affPlotLive;
	long ct;
	int nt;
	double[] txn;
	double[] tft;
	double[] tfg;

    public TCL2() {
		
        // paramtres d'entre
        
		
        // paramtres de sortie
		
        ParamOut dx = new ParamOut();
        dx.type = "nombre";
        dx.label = "dx";
        dx.titre = "";
        addParamOut(dx);
        ParamOut dy = new ParamOut();
        dy.type = "nombre";
        dy.label = "dy";
        dy.titre = "";
        addParamOut(dy);
		
		// Gauss limit
        ParamOut gx = new ParamOut();
        gx.type = "nombre";
        gx.label  = "gx";
        gx.titre = "";
        addParamOut(gx);
        ParamOut gy = new ParamOut();
        gy.type = "nombre";
        gy.label  = "gy";
        gy.titre = "";
        addParamOut(gy);
        
		// infos (n itrations)
        ParamOut info = new ParamOut();
        info.type = "string";
        info.label = "info";
        addParamOut(info);
		
        // options d'affichage
		appletDoc("TCL2_doc.xml");
        
        // affichage "plotlive"
        affPlotLive = new Affichage();
        affPlotLive.type = "plotlive";
        affPlotLive.titre = "TCL";
        String[][] pparams = {{"dx", "dy"},{"gx","gy"}};
        affPlotLive.params = pparams;
		affPlotLive.points = "none";
		affPlotLive.effacer = true;
		affPlotLive.connecter = true;
        addAffichage(affPlotLive);
    }
	
    // calcul live (utilis avec affichage "plotlive")
    // initialisation
    public void initCalculLive(ListeValeurs in) throws SimuException {
		
		double[] boxF = MoreProba.bestBox2(1.0, 1);
		double xmin =-fact;
		double xmax = fact;
		double ymin = boxF[2];
		double ymax = boxF[3];
		
		affPlotLive.xdebut = "" + xmin;
		affPlotLive.xfin = "" + xmax;
		affPlotLive.ydebut = "" + ymin;
		affPlotLive.yfin = "" + ymax;
		
		// Compute reduced normal f.r.
		txn = new double[nGauss];
		tft = new double[nGauss];
		
		double dx = (xmax-xmin)/(nGauss-1);
		for (int i=0; i<nGauss; i++)
			txn[i] = xmin + i*dx;
		
		tfg = Proba.fnormal(0, 1, txn);
		
		nt = 0;
		ct = System.currentTimeMillis();
    }
    
    // Calcul du temps t au temps t+1.
    public ListeValeurs calculLive() throws SimuException {
        if (nt >= numax)
            return null; // fin du calcul
		
		nt++;
		
		double nu = nt;
		
        ListeValeurs out = new ListeValeurs();
		
		double xxmin = Math.max(0.0, Math.floor(nu - fact*Math.sqrt(nu)));
		double xxmax = Math.ceil(nu + fact*Math.sqrt(nu));
		double dx = (xxmax-xxmin)/(nGauss-1);
		double[] tx = new double[nGauss];
		for (int i=0; i<nGauss; i++)
			tx[i] = xxmin + i*dx;
		tft = Proba.fgamma(nu, 1.0, tx);
		for (int i=0; i<nGauss; i++)
			tx[i] = (tx[i]-nu)/Math.sqrt(nu);
		for (int i=0; i<nGauss; i++) {
			out.ajouterDouble("dx", tx[i]);
			out.ajouterDouble("dy", tft[i]);
		}
		for (int i=0; i<nGauss; i++) {
			out.ajouterDouble("gx", txn[i]);
			out.ajouterDouble("gy", tfg[i]);
		}
		
		out.ajouterString("info", "nu = " + nu);
		
		// pour bloquer le programme pendant dt-tempo secondes
		long ct2 = System.currentTimeMillis();
		if (ct2 - ct < tempo) {
	        try {
	            Thread.currentThread().sleep(tempo - (ct2 - ct));
	        } catch (InterruptedException e) {}
			ct += tempo;
		} else
			ct = ct2;
		
        return out;
    }
}
